import { Server } from 'socket.io'
import { TSocket } from '../../../../utilities/types'
import { writeFile } from 'node:fs/promises'
import path from 'path'
import fs from 'fs/promises'
import prisma from '../../../../utilities/prisma'
import sharp from 'sharp'
import logger from '../../../../utilities/logger'
import CharacterManager from '../../../../managers/characterManager'

interface IObjectData {
  [key: string]: Buffer
}

export default class ObjectUploadEvent {
  constructor(
    private readonly io: Server,
    private readonly socket: TSocket
  ) {}

  public listen(): void {
    this.socket.on('gm:object:upload', this.handleObjectUpload.bind(this))
  }

  private async handleObjectUpload(data: IObjectData, callback: (response: boolean) => void): Promise<void> {
    try {
      const character = CharacterManager.getCharacterFromSocket(this.socket);
      if (character?.role !== 'gm') {
        callback(false)
        return
      }

      const public_folder = path.join(process.cwd(), 'public', 'objects')

      // Ensure the folder exists
      await fs.mkdir(public_folder, { recursive: true })

      const uploadPromises = Object.entries(data).map(async ([key, objectData]) => {
        // Get image dimensions
        const metadata = await sharp(objectData).metadata()
        const width = metadata.width || 0
        const height = metadata.height || 0

        const object = await prisma.object.create({
          data: {
            name: key,
            tags: [],
            originX: 0,
            originY: 0,
            frameWidth: width,
            frameHeight: height
          }
        })

        const uuid = object.id
        const filename = `${uuid}.png`
        const finalFilePath = path.join(public_folder, filename)
        await writeFile(finalFilePath, objectData)

        logger.info('gm:object:upload', `Object ${key} uploaded with id ${uuid}`)
      })

      await Promise.all(uploadPromises)

      callback(true)
    } catch (error: any) {
      logger.error('gm:object:upload error', error.message)
      callback(false)
    }
  }
}
